// $Author: tej $
// $Date: 1999/06/20 17:03:59 $
// $Header: /home/tej/drdobbs/RCS/bezier.cpp,v 1.12 1999/06/20 17:03:59 tej Exp $
// $Id: bezier.cpp,v 1.12 1999/06/20 17:03:59 tej Exp $
// $Locker:  $
// $Log: bezier.cpp,v $
// Revision 1.12  1999/06/20 17:03:59  tej
// fiddle with hidden lines
//
// Revision 1.11  1999/06/17 02:30:34  tej
// use name "silhouette"
//
// Revision 1.10  1999/06/12 23:45:55  tej
// streamlining
//
// Revision 1.9  1999/06/12 22:19:49  tej
// backing out C++
//
// Revision 1.8  1999/06/12 21:09:54  tej
// works slow
//
// Revision 1.7  1999/06/12 19:43:39  tej
// works but slow
//
// Revision 1.6  1999/06/12 19:13:59  tej
// removed arrays from room.cpp
//
// Revision 1.5  1999/06/12 03:31:09  tej
// namespaces, cleanup
//
// Revision 1.4  1999/06/09 01:56:25  tej
// working normal edge detection
//
// Revision 1.3  1999/06/08 02:32:24  tej
// started edge normals, forced two-sided  lighting
//
// Revision 1.2  1999/06/06 21:51:14  tej
// includes all differentials and normalization, results uneven
//
// Revision 1.1  1999/06/06 02:57:19  tej
// Initial revision
//
//
// $Name:  $

#include <GL/gl.h>
#include <math.h>
#include <ctype.h>
#include <assert.h>
#include "bezier.h"
#include "roomutil.h"
#include "room.h"

bool isoparm = false, silhouette = false;

namespace {
  const size_t PARM_STEP_QTY = (64);
  double PARM_STEP = 1.0 / (double)PARM_STEP_QTY;
  size_t point_index = MAX_POINTS; 
  GLdouble  newx = 0., newy = 0., newz = 0.;
  int new_control_points = 1;
  void control_ary_to_matrix(bezier_type points,
    GLdouble p_control[DIM_QTY][DIM_QTY][DIM_QTY])
  {
    size_t k, a, b;

    assert(p_control);
    for (k = 0; k < MAX_POINTS; ++k) {
      a = k / DIM_QTY;
      b = k % DIM_QTY; 
      memcpy(p_control[a][b], points[k], sizeof points[k]);
      p_control[a][b][3] = 0.;
    }
  }
  void make_b(GLdouble u, GLdouble b[DIM_QTY])
  {
    double one_minus_u = 1. - u;

    b[0] = pow(one_minus_u, 3.);
    b[1] = 3. * u * pow(one_minus_u, 2.);
    b[2] = 3. * pow(u, 2.) * one_minus_u;
    b[3] = pow(u, 3.);
  }
  void find_p(const GLdouble bu[DIM_QTY], const GLdouble bv[DIM_QTY], 
     GLdouble p_control[DIM_QTY][DIM_QTY][DIM_QTY], GLdouble p[DIM_QTY])
  {
    size_t i, j, k;

    for (i = 0; i < DIM_QTY; ++i) p[i] = 0.;
    for (k = 0; k < DIM_QTY; ++k) {
      for (i = 0; i < DIM_QTY; ++i) {
        for (j = 0; j < DIM_QTY; ++j)
          p[k] += bu[i] * bv[j] * p_control[i][j][k];
      }
    }
  }
  bool oppositesigns(const double a, const double b)
  {
    return ((a * b) < 0);
  }
  double dot(const GLdouble x1[DIM_3D], const GLdouble x2[DIM_3D])
  {
    return x1[0]*x2[0] + x1[1]*x2[1] + x1[2]*x2[2];
  }
};

void bezier(bezier_type points, const dbl_vec_type LookAtEyeV)
{
  size_t u_ctr, v_ctr;
  double u, v;
  GLdouble lookatdelta[DIM_3D];
  GLdouble n[DIM_QTY], n2[DIM_QTY]; 
  static double p_control[DIM_QTY][DIM_QTY][DIM_QTY],
                vertices[PARM_STEP_QTY + 1][PARM_STEP_QTY + 1][DIM_QTY];
  GLdouble bu[DIM_QTY], bv[DIM_QTY];

  control_ary_to_matrix(points, p_control);
  
  if (new_control_points) {
    for (u_ctr = 0; u_ctr <= PARM_STEP_QTY; ++u_ctr) {
      for (v_ctr = 0; v_ctr <= PARM_STEP_QTY; ++v_ctr) {
        u = (double)u_ctr * PARM_STEP;
        v = (double)v_ctr * PARM_STEP;
        make_b(u, bu);
        make_b(v, bv);
        find_p(bu, bv, p_control, vertices[u_ctr][v_ctr]);
      }
    }
    new_control_points = 0;
  }


  if (silhouette || isoparm) 
  {
    glBegin(GL_LINES);
    glDisable(GL_DEPTH_TEST);
  }
  else 
  { 
    glBegin(GL_QUADS);
	}
  for (u_ctr = 0; u_ctr < PARM_STEP_QTY; ++u_ctr) {
    for (v_ctr = 0; v_ctr < PARM_STEP_QTY; ++v_ctr) {
      if (isoparm) {
        if (!(u_ctr % 4) || (u_ctr == (PARM_STEP_QTY - 1))) {
          glVertex3dv(vertices[u_ctr][v_ctr]);
          glVertex3dv(vertices[u_ctr][v_ctr + 1]);
        }
      }else {
        // find the normals for every 
        // neighbouring flap rookwise, not bishop wise.
        // Take (vertex-lookatpoint) * n for each patch
        // If the dot changes sign, 
        // draw a black line on the edge between the two patches.
        // If there is no neighbour, draw the boundary edge.  
        // Let OpenGL do hidden lines.
        // Use this effect in 3D, not in z-buffer display.
        cross(vertices[u_ctr][v_ctr + 1], vertices[u_ctr][v_ctr], 
              vertices[u_ctr + 1][v_ctr], n);
        if (silhouette) {
          if ((!u_ctr) || ((u_ctr + 1) == PARM_STEP_QTY)) { // draw border
            glVertex3dv(vertices[u_ctr][v_ctr]);
            glVertex3dv(vertices[u_ctr][v_ctr + 1]);
          }
          if ((!v_ctr) || ((v_ctr + 1) == PARM_STEP_QTY)) { // draw border
            glVertex3dv(vertices[u_ctr][v_ctr]);
            glVertex3dv(vertices[u_ctr + 1][v_ctr]);
          }
          if ((v_ctr + 2) <= PARM_STEP_QTY) {
            cross(vertices[u_ctr][v_ctr + 2], vertices[u_ctr][v_ctr + 1], 
              vertices[u_ctr + 1][v_ctr + 1], n2);
            lookatdelta[0] = vertices[u_ctr][v_ctr + 1][0] - LookAtEyeV[0];
            lookatdelta[1] = vertices[u_ctr][v_ctr + 1][1] - LookAtEyeV[1];
            lookatdelta[2] = vertices[u_ctr][v_ctr + 1][2] - LookAtEyeV[2];
            if (oppositesigns(dot(lookatdelta, n), dot(lookatdelta, n2))) {
              glVertex3dv(vertices[u_ctr][v_ctr+1]);
              glVertex3dv(vertices[u_ctr + 1][v_ctr+1]);
            }
          }
          if ((u_ctr + 2) <= PARM_STEP_QTY) {
            cross(vertices[u_ctr + 1][v_ctr + 1], vertices[u_ctr + 1][v_ctr], 
              vertices[u_ctr + 2][v_ctr], n2);
            lookatdelta[0] = vertices[u_ctr + 1][v_ctr][0] - LookAtEyeV[0];
            lookatdelta[1] = vertices[u_ctr + 1][v_ctr][1] - LookAtEyeV[1];
            lookatdelta[2] = vertices[u_ctr + 1][v_ctr][2] - LookAtEyeV[2];
            if (oppositesigns(dot(lookatdelta, n), dot(lookatdelta, n2))) {
              glVertex3dv(vertices[u_ctr+1][v_ctr]);
              glVertex3dv(vertices[u_ctr + 1][v_ctr+1]);
            }
          }
        }else {
          glNormal3dv(n);
          glVertex3dv(vertices[u_ctr][v_ctr]);
          glVertex3dv(vertices[u_ctr][v_ctr + 1]);
          glVertex3dv(vertices[u_ctr + 1][v_ctr + 1]);
          glVertex3dv(vertices[u_ctr + 1][v_ctr]);
        }
      }
    }
  }
  glEnd();

#if 0
  glBegin(GL_QUADS);
  if (isoparm || silhouette) {
    GLfloat not_specular[] = {.7f, .7f, .7f, 1.f};
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 0.0);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, not_specular);
    glDisable(GL_DEPTH_TEST);
	}
  for (u_ctr = 0; u_ctr < PARM_STEP_QTY; ++u_ctr) {
    for (v_ctr = 0; v_ctr < PARM_STEP_QTY; ++v_ctr) {
        cross(vertices[u_ctr][v_ctr + 1], vertices[u_ctr][v_ctr], 
              vertices[u_ctr + 1][v_ctr], n);
          glNormal3dv(n);
          glVertex3dv(vertices[u_ctr][v_ctr]);
          glVertex3dv(vertices[u_ctr][v_ctr + 1]);
          glVertex3dv(vertices[u_ctr + 1][v_ctr + 1]);
          glVertex3dv(vertices[u_ctr + 1][v_ctr]);
    }
  }
  glEnable(GL_DEPTH_TEST);
  glEnd();
#endif


  glFlush();
}





