import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.multimedia.*;

/**
 * This class reads PARAM tags from its HTML host page and sets
 * the color and label properties of the applet. Program execution
 * begins with the init() method. 
 */
public class NonOptimizedDemo extends Applet implements Runnable
{
  Thread                mainThread;
  MediaTracker          tracker;
  NonOptimizedSaucerMgr saucerMgr;
  Image                 dblBuffer;
  Graphics              dblBufferGraphics;
  String                performanceString;
  int                   saucerEvent = 0;
  Object                synchObject = new Object();
  int                   fps = 0;
  FrameRateProfiler     frp = new FrameRateProfiler();
  
  static final int FRAME_DELAY = 33; // ms, or 30 fps max
    
	/**
	 * The entry point for the applet. 
	 */
	public void init()
	{
    tracker = new MediaTracker(this);
    java.net.URL base = getDocumentBase();
    String imageStr = "saucers.gif";
    if (base.toString().indexOf("Demo") < 0)
      imageStr = "Demo\\" + imageStr;
    Image saucerImages = getImage(getDocumentBase(), imageStr);
    tracker.addImage(saucerImages,0);
    saucerMgr = new NonOptimizedSaucerMgr(this, saucerImages);
    initForm();
    dblBuffer = createImage(getBounds().width, getBounds().height);
    dblBufferGraphics = dblBuffer.getGraphics();
	}

	Label label1 = new Label();
  Panel status = new Panel();
  Button removeButton = new Button("-");
  Button addButton    = new Button("+");

  /**
	 * Intializes values for the applet and its components
	 */
	void initForm()
	{
		this.setBackground(Color.black);
		this.setForeground(Color.lightGray);
		label1.setText("label1                       ");
    label1.setBackground(Color.black);
		label1.setForeground(Color.white);
		this.setLayout(new BorderLayout());
    status.setBackground(Color.black);
    status.setForeground(Color.lightGray);
    status.setLayout(new FlowLayout());
    status.add(label1);
    status.add(removeButton);
    status.add(addButton);
    this.add("North", status);
    
    removeButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          synchronized (synchObject) {
            --saucerEvent;
          }
          fps = 0;
        }
      }
    );
    addButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          synchronized (synchObject) {
            ++saucerEvent;
          }
          fps = 0;
        }
      }
    );

	}
  
  public void update(Graphics g) {
    paint(g);
  }
  
  public void paint(Graphics g) {
    dblBufferGraphics.setColor(getBackground());
    dblBufferGraphics.fillRect(0,0,getBounds().width,getBounds().height);
    saucerMgr.drawAll(dblBufferGraphics);
    g.drawImage(dblBuffer,0,0,null);
    frp.mark();
    int tmp = frp.getProfileInt(true);
    if (fps != tmp) {
      label1.setText("saucers:" + saucerMgr.size() + "; fps:" + tmp);
      fps = tmp;
    }
  }
  
  public void start() {
    frp.start(750);
    if (mainThread == null)
      mainThread = new Thread(this);
    mainThread.start();
  }
  
  public void stop() {
    if (mainThread != null)
      mainThread = null;
    frp.stop();
  }
  
  public void run() {
    try {
      tracker.waitForID(0);
      
    } catch (InterruptedException e) {
      return;
    }
                                       
    while (mainThread != null) {
      
      // Apply add/remove events to next rendering frame
      int tmp;
      synchronized (synchObject) {
        tmp = saucerEvent;
        saucerEvent= 0;
      }
      if (tmp > 0) {
        for (int i = 0; i < tmp; ++i)
          saucerMgr.add(true);
      } else if (tmp < 0) {
        for (int j = 0; j < Math.abs(tmp); ++j)
          saucerMgr.remove();
      }
      saucerMgr.updateAll();

      
      repaint();
      Thread.yield();
      try {
        Thread.sleep(FRAME_DELAY);
      } catch (Exception e) {};
    }
  }
}
