// $Author: tej $
// $Date: 1999/06/19 15:12:22 $
// $Header: /home/tej/drdobbs/RCS/room.cpp,v 1.21 1999/06/19 15:12:22 tej Exp $
// $Id: room.cpp,v 1.21 1999/06/19 15:12:22 tej Exp $
// $Locker:  $
// $Log: room.cpp,v $
// Revision 1.21  1999/06/19 15:12:22  tej
// fix menus and shininess of bezier
//
// Revision 1.20  1999/06/18 03:12:28  tej
// fix most normals
//
// Revision 1.19  1999/06/17 02:30:34  tej
// use name "silhouette"
//
// Revision 1.18  1999/06/14 02:00:13  tej
// changed a couple names
//
// Revision 1.17  1999/06/13 00:27:09  tej
// tweaked threshhold
//
// Revision 1.16  1999/06/12 23:45:55  tej
// streamlining
//
// Revision 1.15  1999/06/12 22:43:51  tej
// separate laplce
//
// Revision 1.14  1999/06/12 22:37:25  tej
// sobel_diff
//
// Revision 1.13  1999/06/12 22:25:07  tej
// if first matrix setup
//
// Revision 1.12  1999/06/12 19:43:39  tej
// works but slow
//
// Revision 1.11  1999/06/12 19:22:27  tej
// work s again
//
// Revision 1.10  1999/06/12 19:13:59  tej
// removed arrays from room.cpp
//
// Revision 1.9  1999/06/12 03:31:09  tej
// namespaces, cleanup
//
// Revision 1.8  1999/06/09 01:56:41  tej
// working normal edge detection
//
// Revision 1.7  1999/06/06 23:56:25  tej
// moved out objects
//
// Revision 1.6  1999/06/06 21:51:14  tej
// includes all differentials and normalization, results uneven
//
// Revision 1.5  1999/06/06 02:56:48  tej
// link to bezier and roomutil
//
// Revision 1.4  1999/06/05 19:09:58  tej
// misc. changes
// remove comments
//
// Revision 1.3  1999/06/05 16:30:38  tej
// new menu, strip down keybd commands
//
// Revision 1.2  1999/06/05 15:32:35  tej
// removed dresser, file, headlight, moved furniture down towards end.
//
// Revision 1.1  1999/06/05 15:02:27  tej
// Initial revision
//
// $Name:  $
// Thomas E. Janzen
// Room
// March 30 1998 / April 4 1998
#include <GL/glut.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <vector>

#include "bezier.h"
#include "roomutil.h"
#include "world.h"
#include "room.h"

enum outliners {none, sobel, laplace};

GLdouble LookAtEye[DIM_3D] = {60., 73., -53};
const GLsizei SCREENW = 500, SCREENH = 500;

namespace {
  dbl_vec_type LookAtEyeV(LookAtEye, &LookAtEye[DIM_3D]);
  GLdouble 
    LookAtCenter[] = {208., 72., -70.},
    LookAtDirection[] = {1, 0, 0},
    LookAtUp[] = {0., 1., 0.};
  dbl_vec_type LookAtCenterV(LookAtCenter, &LookAtCenter[DIM_3D]),
                 LookAtDirectionV(LookAtDirection, &LookAtDirection[DIM_3D]),
                 LookAtUpV(LookAtUp, &LookAtUp[DIM_3D]);
  double LookAtAngle = 1.57;
  bool z_on = false;
  GLsizei gw = SCREENW, gh = SCREENH;
  const size_t OPW = 3;
  enum outliners outliner = none;
  void NormalizeAngle(double *angle)
  {
    while (*angle < -(2 * M_PI)) {
      *angle += (2 * M_PI);
    }
    while (*angle > 2 * M_PI) {
      *angle -= (2 * M_PI);
    }
  }

  void update_direction(void)
  {
    NormalizeAngle(&LookAtAngle);
    LookAtDirectionV[0] = sin(LookAtAngle);
    LookAtDirectionV[1] = 0;
    LookAtDirectionV[2] = cos(LookAtAngle);
  }

  void sobel_diff(GLfloat pixels[SCREENW][SCREENH])
  {
    static GLfloat diffpixels[SCREENW][SCREENH];
    GLfloat suma, sumb;
    GLsizei i, j;
    int io, jo;
    GLfloat sobela[OPW][OPW] = {{ 1, 2, 1}, { 0, 0, 0}, {-1, -2, -1}},
            sobelb[OPW][OPW] = {{-1, 0, 1}, {-2, 0, 2}, {-1,  0,  1}};
    static flt_mat_type sobelaM(OPW), sobelbM(OPW);
    bool first = true;
    if (first) {
      for (size_t i = 0; i < OPW; ++i) {
        for (size_t j = 0; j < OPW; ++j) {
          sobelaM[i].resize(OPW);
          sobelaM[i][j] = sobela[i][j];
          sobelbM[i].resize(OPW);
          sobelbM[i][j] = sobelb[i][j];
        }
      }
      first = false;
    }
    for (i = 1; i < (gw - 1); ++i) {
      for (j = 1; j < (gh - 1); ++j) {
        suma = 0.0; sumb = 0;
        for (io = -1; io <= 1; ++io) {
          for (jo = -1; jo <= 1; ++jo) {
            suma  += sobelaM[io+1][jo+1] * pixels[i + io][j + jo];
            sumb += sobelbM[io+1][jo+1] * pixels[i + io][j + jo];
          }
        }
        diffpixels[i][j] = (fabs(suma) + fabs(sumb)) / 8.0;
      }
    }
    memcpy(pixels, diffpixels, sizeof diffpixels);
  }

  void laplace_diff(GLfloat pixels[SCREENW][SCREENH])
  {
    static GLfloat diffpixels[SCREENW][SCREENH];
    static GLfloat laplaceline[OPW][OPW]
       = {{-1, -1, -1}, {-1, 8, -1}, {-1, -1, -1}};
    static flt_mat_type laplacelineM(OPW);
    GLfloat sumo;
    GLsizei i, j;
    bool first = true;
    int io, jo;

    if (first) {
      for (size_t i = 0; i < OPW; ++i) {
        for (size_t j = 0; j < OPW; ++j) {
          laplacelineM[i].resize(OPW);
          laplacelineM[i][j] = laplaceline[i][j];
        }
      }
      first = false;
    }
    for (i = 1; i < (gw - 1); ++i) {
      for (j = 1; j < (gh - 1); ++j) {
        sumo = 0.0;
        for (io = -1; io <= 1; ++io)
          for (jo = -1; jo <= 1; ++jo)
            sumo  += laplaceline[io+1][jo+1] * pixels[i + io][j + jo];
        diffpixels[i][j] = sumo / 3;
      }
    }
    memcpy(pixels, diffpixels, sizeof diffpixels);
  }

  void display(void)
  {
    static GLfloat pixels[SCREENW][SCREENH], dmin, dmax, drange, dscale;
    GLsizei i, j;

    glEnable(GL_LIGHT0);
    glClearColor((float).6, (float).6, (float)1., (float)0.);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(90., 1., 1., 300.);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    for (size_t ctr = 0; ctr < 3; ++ctr) {
      LookAtCenterV[ctr] = LookAtEyeV[ctr] + LookAtDirectionV[ctr];
    }
    gluLookAt(LookAtEyeV[0], LookAtEyeV[1], LookAtEyeV[2],
              LookAtCenterV[0], LookAtCenterV[1], LookAtCenterV[2],
              LookAtUpV[0], LookAtUpV[1], LookAtUpV[2]);
    glMatrixMode(GL_MODELVIEW);

    draw_room(LookAtEyeV);

    glFlush();
    if (z_on) {
      GLfloat threshhold = (GLfloat)0.5;
      glReadBuffer(GL_BACK);
      glDrawBuffer(GL_BACK);
      glReadPixels(0, 0, gw, gh, GL_DEPTH_COMPONENT, GL_FLOAT, pixels);

      switch(outliner) {
        case sobel:
          sobel_diff(pixels);
          threshhold = .11;
          break;
        case laplace:
          laplace_diff(pixels);
          threshhold = .483;
          break;
        default:
          break;
      }
      if (outliner != none) {
        dmax = dmin = pixels[0][0];
        for (i = 1; i < (gw - 1); ++i) {
          for (j = 1; j < (gh - 1); ++j) {
            if (pixels[i][j] > dmax) dmax = pixels[i][j];
            if (pixels[i][j] < dmin) dmin = pixels[i][j];
          }
        }
        drange = dmax - dmin;
        if (drange) dscale = 1.0 / drange;
        else dscale = 1.0;
        // clamp
        for (i = 1; i < gw - 1; ++i) {
          for (j = 1; j < gh - 1; ++j) {
            pixels[i][j] = (pixels[i][j] - dmin) * dscale;
            pixels[i][j] = ((pixels[i][j] >= threshhold) ? 0: 0.999990);
          }
        }
      }
      else {
        dmax = dmin = pixels[0][0];
        for (i = 1; i < (gw - 1); ++i) {
          for (j = 1; j < (gh - 1); ++j) {
            if (pixels[i][j] > dmax) dmax = pixels[i][j];
            if (pixels[i][j] < dmin) dmin = pixels[i][j];
          }
        }
        drange = dmax - dmin;
        if (drange) dscale = 1.0 / drange;
        else dscale = 1.0;
        for (i = 1; i < gw - 1; ++i)
          for (j = 1; j < gh - 1; ++j) 
            pixels[i][j] = (pixels[i][j] - dmin) * dscale;
      }
      glDrawPixels(gw, gh, GL_LUMINANCE, GL_FLOAT, pixels);
    }
    glutSwapBuffers();
  }

  void pan_menu_func(const int item)
  {
    const double angle_step 
      = ((1 == item) ? -(M_PI / 180. * 20.) : (M_PI / 180. * 20.));

    LookAtAngle += angle_step;
    update_direction();
    display();
  }

  void truck_menu_func(const int item)
  {
    const double local_step = ((1 == item) ? 10 : -10);

    LookAtEyeV[0] += local_step * LookAtDirectionV[0];
    LookAtEyeV[1] += local_step * LookAtDirectionV[1];
    LookAtEyeV[2] += local_step * LookAtDirectionV[2];
    display();
  }

  void outline_menu_func(const int item)
  {
    switch (item) {
      case 1:
        outliner = none;
        z_on = true;
        break;
      case 2:
        outliner = sobel;
        z_on = true;
        break;
      case 3:
        outliner = laplace;
        z_on = true;
        break;
      default:
        break;
    }
    display();
  }

  void effects_menu_func(const int item)
  {
    switch (item) {
      case 1:
        z_on = !z_on;
        break;
      case 3:
        isoparm = !isoparm;
        if (isoparm) silhouette = false;
        break;
      case 4:
        silhouette = !silhouette;
        if (silhouette) isoparm = false;
        break;
      default:
        break;
    }
    display();
  }

  void bezier_menu_func(const int item)
  {
    switch (item) {
      case 1:
        isoparm = false;
        silhouette = false;
        break;
      case 2: 
        isoparm = true;
        silhouette = false;
        break; 
      case 3:
        isoparm = false;
        silhouette = true;
        break;
      default:
        break;
    }
    display();
  }

  void render_menu_func(const int item)
  {
    switch (item) {
      case 2:
        z_on = false;
        break;
      default:
        break;
    }
    display();
  }

  void main_menu_func(const int item)
  {
    switch (item) {
      case 3:
        exit(EXIT_SUCCESS);
        break;
      default:
        break;
    }
  }

  void init(void)
  {
    int main_menu_id, camera_menu_id, pan_menu_id, truck_menu_id, 
        render_menu_id, bezier_menu_id, z_menu_id;

    pan_menu_id = glutCreateMenu(pan_menu_func); /* ok*/ 
    glutAddMenuEntry("right", 1);
    glutAddMenuEntry("left",  2);

    truck_menu_id = glutCreateMenu(truck_menu_func); /*ok*/
    glutAddMenuEntry("forward",  1);
    glutAddMenuEntry("backward", 2);

    z_menu_id = glutCreateMenu(outline_menu_func); /*ok*/
    glutAddMenuEntry("none", 1);
    glutAddMenuEntry("sobel", 2);
    glutAddMenuEntry("laplace", 3);

    bezier_menu_id = glutCreateMenu(bezier_menu_func); /* ok */
    glutAddMenuEntry("patches", 1);
    glutAddMenuEntry("isoparametric", 2);
    glutAddMenuEntry("silhouette", 3);

    render_menu_id = glutCreateMenu(render_menu_func);
    glutAddSubMenu("Z outlines", z_menu_id);
    glutAddMenuEntry("polygons", 2);

    camera_menu_id = glutCreateMenu(0);
    glutAddSubMenu("pan", pan_menu_id);
    glutAddSubMenu("truck", truck_menu_id);

    main_menu_id = glutCreateMenu(main_menu_func);
    glutAttachMenu(GLUT_RIGHT_BUTTON);
    glutAddSubMenu("camera", camera_menu_id);
    glutAddSubMenu("render", render_menu_id);
    glutAddSubMenu("Bezier", bezier_menu_id);
    glutAddMenuEntry("EXIT", 3);

    glEnable(GL_LIGHTING);
    glEnable(GL_NORMALIZE);
    glEnable(GL_DEPTH_TEST);
  }

  void reshape(GLsizei w, GLsizei h)
  {
    display();
    glFlush();
  }
}

int main(int argc, char** argv)
{
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutInitWindowSize(SCREENW, SCREENH);
  glutCreateWindow("Room");
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(90., 1., 1., 300.);
  glMatrixMode(GL_MODELVIEW);
  glutDisplayFunc(display);

  glutReshapeFunc(reshape);
  init();
  glutMainLoop();
  return EXIT_SUCCESS;
}

