// $Author: tej $
// $Date: 1999/06/20 17:03:59 $
// $Header: /home/tej/drdobbs/RCS/world.cpp,v 1.10 1999/06/20 17:03:59 tej Exp $
// $Id: world.cpp,v 1.10 1999/06/20 17:03:59 tej Exp $
// $Locker:  $
// $Log: world.cpp,v $
// Revision 1.10  1999/06/20 17:03:59  tej
// fiddle with hidden lines
//
// Revision 1.9  1999/06/19 15:12:22  tej
// fix menus and shininess of bezier
//
// Revision 1.8  1999/06/18 03:12:28  tej
// fix most normals
//
// Revision 1.7  1999/06/12 23:45:55  tej
// streamlining
//
// Revision 1.6  1999/06/12 21:09:54  tej
// works slow
//
// Revision 1.5  1999/06/12 19:43:39  tej
// works but slow
//
// Revision 1.4  1999/06/12 19:13:59  tej
// removed arrays from room.cpp
//
// Revision 1.3  1999/06/12 03:31:09  tej
// namespaces, cleanup
//
// Revision 1.2  1999/06/08 02:32:15  tej
// started edge normals
//
// Revision 1.1  1999/06/06 23:56:04  tej
// Initial revision
//
// $Name:  $


#include <GL/glut.h>
#include <fstream>
#include <cstdlib>

#include "bezier.h"
#include "world.h"
#include "roomutil.h"
#include "room.h"

namespace {
  GLfloat
    chair_diffuse[]  = {1.f, .5f, 0.f, 1.0f},
    globe_diffuse[]  = {.1f, .9f, .9f, 1.f},
    globe_specular[] = {.5f, .5f, .5f, 1.f},
    bezier_specular[] = {.7f, .7f, .7f, 1.f},
    bezier_diffuse[] = {.3f, .3f, 0.f, 1.f},
    ambient[] = {.0f, .0f, .0f, 1.f},
    desk_specular[]  = {.3f, .3f, .3f, 1.f},
    desk_diffuse[]   = {1.f, .73f, .53f, 1.f},
    wall_specular[]  = {.0f, .0f, .0f, 1.f},
    overhead[] = {104.f, 90.f, -79.f, 1.f}  ,
    overhead_diffuse[] = {1.f, 1.f, 1.f, 1.f}, 
    overhead_specular[] = {1.f, 1.f, 1.f, 1.f},
    wall_diffuse[]   = {1.f, .87f, .73f, 1.f};
  GLdouble
    room[][3] = {
     {0,  0, -50}, {0,  0, 0}, {57,  0, 0}, {208,  0, 0}, {208,  0, -137}, 
     {0, 0, -137},
     {0,  93, -50}, {0,  93, 0}, {57,  93, 0}, {208,  93, 0}, {208,  93, -137}, 
     {0, 93, -137}},
    walls[][3] = {
     {0,  0, -50}, {57,  0, -50}, {57,  0, 0}, {208,  0, 0}, {208,  0, -137}, 
     {0, 0, -137},
     {0,  93, -50}, {57, 93, -50}, {57,  93, 0}, {208, 93, 0}, {208,  93, -137}, 
     {0, 93, -137}},
    desk[] = {60, 30, 30.5};

  void readbezierpoints(bezier_type points)
  {
    size_t pts_ctr, num_per_pt_ctr;
    GLdouble temp = 0.0;
          
    std::ifstream bezierfile("bezier.dat");

    for (pts_ctr = 0; pts_ctr < MAX_POINTS;
       ++pts_ctr)
    {
        for (num_per_pt_ctr = 0; num_per_pt_ctr < DIM_QTY;
           ++num_per_pt_ctr)
        {
          bezierfile >> temp;
          points[pts_ctr][num_per_pt_ctr] = temp;
        }
    }
  }

  void tjRect3dv(const GLdouble *v1, const GLdouble *v2)
  {
    GLdouble a[3], b[3], c[3], d[3];

    if (v1[0] == v2[0]) {
      a[0] = v1[0]; a[1] = v1[1]; a[2] = v1[2];
      b[0] = v1[0]; b[1] = v2[1]; b[2] = v1[2];
      c[0] = v1[0]; c[1] = v2[1]; c[2] = v2[2];
      d[0] = v1[0]; d[1] = v1[1]; d[2] = v2[2];
    }
    else {
      if (v1[1] == v2[1]) {
        a[0] = v1[0]; a[1] = v1[1]; a[2] = v1[2];
        b[0] = v2[0]; b[1] = v1[1]; b[2] = v1[2];
        c[0] = v2[0]; c[1] = v1[1]; c[2] = v2[2];
        d[0] = v1[0]; d[1] = v1[1]; d[2] = v2[2];
      }
      else {
        if (v1[2] == v2[2]) {
          a[0] = v1[0]; a[1] = v1[1]; a[2] = v1[2];
          b[0] = v2[0]; b[1] = v1[1]; b[2] = v1[2];
          c[0] = v2[0]; c[1] = v2[1]; c[2] = v1[2];
          d[0] = v1[0]; d[1] = v2[1]; d[2] = v1[2];
        }
      }
    }
    glBegin(GL_POLYGON);
    glVertex3dv(a);
    glVertex3dv(b);
    glVertex3dv(c);
    glVertex3dv(d);
    glEnd();
  }

  void draw_box(const GLdouble *Vin)
  {
    GLdouble w1[3], w2[3], w3[3], n[3];

    w1[0] = w1[1] = w1[2] = 0;
    w2[0] = Vin[0]; w2[1] = Vin[1]; w2[2] = 0;
    w3[0] = 0; w3[1] = Vin[1]; w3[2] = 0;
    cross(w1, w3, w2, n);
    glNormal3dv(n);
    tjRect3dv(w1, w2);

    w2[0] = 0; w2[2] = Vin[2];
    w3[0] = 0; w3[1] = 0; w3[2] = Vin[2];
    cross(w1, w3, w2, n);
    glNormal3dv(n);  
    tjRect3dv(w1, w2);

    w2[0] = Vin[0]; w2[1] = 0; w2[2] = Vin[2];
    w3[0] = Vin[0]; w3[1] = 0; w3[2] = 0;
    cross(w2, w3, w1, n);
    glNormal3dv(n);
    tjRect3dv(w1, w2);

    w1[0] = w1[1] = 0; w1[2] = Vin[2];
    w2[0] = Vin[0]; w2[1] = Vin[1]; w2[2] = Vin[2];
    w3[0] = Vin[0]; w3[1] = 0; w3[2] = Vin[2];
    cross(w2, w3, w1, n);
    glNormal3dv(n);
    tjRect3dv(w1, w2);

    w1[0] = Vin[0]; w1[2] = 0;
    w3[0] = Vin[0]; w3[1] = Vin[1]; w3[2] = 0;
    cross(w2, w3, w1, n);
    glNormal3dv(n);
    tjRect3dv(w1, w2);

    w1[0] = 0; w1[1] = Vin[1]; w1[2] = 0;
    w3[0] = 0; w3[1] = Vin[1]; w3[2] = Vin[2];
    cross(w1, w3, w2, n);
    glNormal3dv(n);
    tjRect3dv(w1, w2);
  }

  void draw_chair(void)
  { 
    GLdouble back[] = {9, 27, 29}, seat[] = {19, 9, 23}, arm[] = {19, 14, 3};

    glPushMatrix();
    glTranslated(9, 8, -3);
    draw_box(arm);
    glPopMatrix();

    glPushMatrix();
    glTranslated(9, 8, -29);
    draw_box(arm);
    glPopMatrix();

    glPushMatrix();
    glTranslated(9, 8, -26);
    draw_box(seat);
    glPopMatrix();

    glPushMatrix();
    glTranslated(0, 8, -29);
    draw_box(back);
    glPopMatrix();
  }

  void draw_room_contents(const dbl_vec_type LookAtEyeV)
  {
    bezier_type points;
    glPushMatrix();
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, desk_specular);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, desk_diffuse);
    glTranslated(118, .1, -130);
    draw_box(desk);
    glPopMatrix();

    glPushMatrix();
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, chair_diffuse);
    glTranslated(190, 0.1, -38);
    glRotated(140.0, 0, 1.0, 0);
    draw_chair();
    glPopMatrix();

    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, chair_diffuse);
    glPushMatrix();
    glTranslated(0, 0, 50);

    readbezierpoints(points);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, bezier_diffuse);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, bezier_specular);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 6.0);
    bezier(points, LookAtEyeV);
    glPopMatrix();
    glPushMatrix();
    glTranslated(165, 39.1, -122);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 6.0);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, globe_diffuse);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, globe_specular);
    GLUquadricObj *globe = gluNewQuadric();
    gluSphere(globe, 8.L, 32, 32);
    glPopMatrix();
  }
}

void draw_room(const dbl_vec_type LookAtEyeV)
{
  GLdouble n[3];
  size_t vert;

  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,  ambient);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,  wall_diffuse);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, wall_specular);
  glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 1.0);

  glLightfv(GL_LIGHT0, GL_POSITION, overhead);
  glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, overhead_diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, overhead_specular);

  glBegin(GL_POLYGON);
  cross(room[4], room[3], room[2], n);
  glNormal3dv(n);
  for (vert = 0; vert < 6; ++vert) {
    glVertex3dv(room[vert]);
  }
  glEnd();

  cross(room[10], room[9], room[8], n);
  glNormal3dv(n);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,  wall_diffuse);
  glBegin(GL_POLYGON);
  for (vert = 6; vert < 12; ++vert) {
    glVertex3dv(room[vert]);
  }
  glEnd();

  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, wall_diffuse);
  cross(walls[7], walls[1], walls[0], n); 
  glNormal3dv(n); 
  tjRect3dv(walls[0], walls[7]);
  cross(walls[1], walls[2], walls[8], n);
  glNormal3dv(n); 
  tjRect3dv(walls[1], walls[8]);
  cross(walls[9], walls[3], walls[2], n); 
  glNormal3dv(n); 
  tjRect3dv(walls[2], walls[9]);
  cross(walls[10], walls[9], walls[3], n); 
  glNormal3dv(n);
  tjRect3dv(walls[10], walls[3]);
  cross(walls[11], walls[5], walls[4], n); 
  glNormal3dv(n);
  tjRect3dv(walls[4], walls[11]);
  cross(walls[5], walls[0], walls[6], n); 
  glNormal3dv(n); 
  tjRect3dv(walls[5], walls[6]);

  draw_room_contents(LookAtEyeV);
}




