package java.multimedia;

public class FrameRateProfiler extends Object implements Runnable
{
  Thread profileThread;
  int    fps = 0;               // Current period fps
  double dfps = 0.0;            // Current period fps
  int    daccuracy = 1;         // # of decimal places
  long   periodLength = 300;    // Period length in millisecond
  long   startTime;             // Profile start time
  long   stopTime;              // Profile stop time
  int    framesThisPeriod = 0;  // # of frames per sampling period
  int    framesThisProfile = 0; // # o frames from start to stop.
  
  public FrameRateProfiler()
  {
  }
  
  public int getProfileInt(boolean bPeriod) {
    if (bPeriod == true)
      return fps;
    else
      return (framesThisProfile*1000/(int)(stopTime-startTime));
  }
  public double getProfileDouble(boolean bPeriod) {
    if (bPeriod == true)
      return (dfps);
    else
      return (1000.0*(double)framesThisProfile)/(double)(stopTime-startTime);
  }
  public void setAccuracy(int decimalPlaces) {
    daccuracy = Math.min(4, Math.abs(decimalPlaces));
  }
  
  public String getProfileString(boolean bPeriod) {
    if (bPeriod == true)
      return ("" + fps);
    else
      return ("" + (framesThisProfile*1000/(stopTime-startTime)));
  }

  public void start(int samplePeriod) {
    periodLength = Math.max(samplePeriod,33);
    if (profileThread == null)
      profileThread = new Thread(this);
    profileThread.start();
  }
  
  public void stop() {
    stopTime = new java.util.Date().getTime();
    if (profileThread != null)
      profileThread = null;
  }
  
  private void updateProfile(long currentTime) {
    dfps = (double)((10^daccuracy)*1000*framesThisPeriod/(int)(currentTime-stopTime))/(double)(10^daccuracy);
    fps = (int)dfps;
    stopTime = currentTime;
    framesThisPeriod=1;
  }
  
  public int mark() {
    ++framesThisPeriod;
    return ++framesThisProfile;
  }
  
  public void run() {
    startTime = new java.util.Date().getTime();
    stopTime = startTime + 1; // avoids divide by zero in getProfile*()
    while (profileThread != null) {
      updateProfile(new java.util.Date().getTime());
      Thread.yield();
      try {
        Thread.sleep(periodLength);
      } catch (Exception e) {};
    }
  }
}
