package java.multimedia;

import java.awt.*;
import java.multimedia.animationSequencer;

public class Sprite extends Object {
  private Component component;        // retrieve graphics class info
  private Image     imageBuffer;      // Current frame to display & recycle object
  private Graphics  bufferGraphics;
  private Image     images;           // All the frames of this sprite.
  Rectangle position;         // x,y,width,height
  Point     prevPos;          // For minimal erasing
  double    vx,vy;            // speed in # pixels per millisecond
  animationSequencer animSeq; // Animation sequence logic
  long               lastUpdateTime;
  Point              boundary = new Point();
  public boolean expired = false;
  
  public Sprite(animationSequencer as, int width, int height) {
    position = new Rectangle();
    position.width=width;
    position.height=height;
    prevPos = new Point(0,0);
    lastUpdateTime = (new java.util.Date()).getTime();
    animSeq = as;
  }
  
  public void reset(Component comp, Image imageSrc, int posx, int posy, double ovx, double ovy) {
    component = comp;
    boundary.x = comp.getBounds().width;
    boundary.y = comp.getBounds().height;
    images = imageSrc;
    position.x = posx;
    position.y = posy;
    vx  = (ovx==0.0)?0.5:ovx;
    vy = (ovy==0.0)?0.5:ovy;
    expired = false;
    if (imageBuffer==null) {
      imageBuffer = component.createImage(position.width,position.height);
      bufferGraphics = imageBuffer.getGraphics();
    }
  }
  
  public void update(long updateTime) {
    int delta = (int)(updateTime - lastUpdateTime);
    if (delta > 0) {
      position.x += ((int)Math.round(vx * delta));
      position.y += ((int)Math.round(vy * delta));
      if ((position.x < 1-position.width) || (position.x > boundary.x) ||
          (position.y < 1-position.height) || (position.y > boundary.y))
        expired = true;   
      
      // Update timer only for noticable movements, 
      if ((position.x != prevPos.x) || (position.y != prevPos.y)) {
        lastUpdateTime = updateTime;
      }
    }
    if (! expired) {
      //g.clipRect(0,0,position.width,position.height);
      bufferGraphics.drawImage(images,-(animSeq.incrementFrame(null)*position.width),0,component);
    }
  }
  
  public void draw(boolean bUpdateFirst, Graphics g) {
    if (bUpdateFirst)
      update(new java.util.Date().getTime());
    if ((position.x != prevPos.x) || (position.y != prevPos.y)) {
      //g.setColor(component.getBackground());
      //g.fillRect(prevPos.x,prevPos.y,position.width,position.height);
      prevPos.x = position.x;
      prevPos.y = position.y;
    }
    if (! expired)
      g.drawImage(imageBuffer, position.x, position.y, component);
  }
}